@@ -9,10 +9,12 @@ import os
 import sys
 import json
 
-from io import BytesIO, StringIO
+from io import IOBase, BytesIO, StringIO
+from typing import Callable, BinaryIO, IO, Iterator, Union
 
+from pymarc.constants import END_OF_RECORD
 from pymarc import Record, Field
-from pymarc.exceptions import PymarcException, RecordLengthInvalid
+from pymarc import exceptions
 
 
 class Reader:
@@ -70,21 +72,15 @@ class MARCReader(Reader):
     if you have a file in incorrect encode and you know what it is, you can
     try to use your encode in parameter "file_encoding".
 
-    You may want to parse data in a permissive way to avoid stopping on the first
-    wrong record and read as much records as possible:
+    MARCReader parses data in a permissive way and gives the user full control
+    on what to do in case wrong record is encountered. Whenever any error is
+    found reader returns ``None`` instead of regular record object.
+    The exception information and corresponding data are available through
+    reader.current_exception and reader.current_chunk properties:
 
     .. code-block:: python
 
-        reader = MARCReader(open('file.dat', 'rb'), permissive=True)
-
-    In such case ``None`` is return by the iterator.
-    This give you the full control to implement the expected behavior getting
-    exception information under ``reader.last_exception`` which will store
-    a tuple with (<chunk_data>, <catched exception>):
-
-    .. code-block:: python
-
-        reader = MARCReader(open('file.dat', 'rb'), permissive=True)
+        reader = MARCReader(open('file.dat', 'rb'))
         for record in reader:
             if record is None:
                 print(
@@ -100,6 +96,8 @@ class MARCReader(Reader):
     _current_chunk = None
     _current_exception = None
 
+    file_handle: IO
+
     @property
     def current_chunk(self):
         """Current chunk."""
@@ -112,14 +110,14 @@ class MARCReader(Reader):
 
     def __init__(
         self,
-        marc_target,
-        to_unicode=True,
-        force_utf8=False,
-        hide_utf8_warnings=False,
-        utf8_handling="strict",
-        file_encoding="iso8859-1",
-        permissive=False,
-    ):
+        marc_target: Union[BinaryIO, bytes],
+        to_unicode: bool = True,
+        force_utf8: bool = False,
+        hide_utf8_warnings: bool = False,
+        utf8_handling: str = "strict",
+        file_encoding: str = "iso8859-1",
+        permissive: bool = False,
+    ) -> None:
         """The constructor to which you can pass either raw marc or a file-like object.
 
         Basically the argument you pass in should be raw MARC in transmission format or
@@ -132,35 +130,52 @@ class MARCReader(Reader):
         self.utf8_handling = utf8_handling
         self.file_encoding = file_encoding
         self.permissive = permissive
-        if hasattr(marc_target, "read") and callable(marc_target.read):
-            self.file_handle = marc_target
-        else:
+        if isinstance(marc_target, bytes):
             self.file_handle = BytesIO(marc_target)
+        else:
+            self.file_handle = marc_target
 
-    def close(self):
+    def close(self) -> None:
         """Close the handle."""
-        if self.file_handle:
-            self.file_handle.close()
-            self.file_handle = None
+        self.file_handle.close()
 
     def __next__(self):
-        first5 = self.file_handle.read(5)
+        """Read and parse the next record."""
+        if self._current_exception:
+            if isinstance(self._current_exception, exceptions.FatalReaderEror):
+                raise StopIteration
+
+        self._current_chunk = None
+        self._current_exception = None
+
+        self._current_chunk = first5 = self.file_handle.read(5)
         if not first5:
             raise StopIteration
+
         if len(first5) < 5:
-            raise RecordLengthInvalid
+            self._current_exception = exceptions.TruncatedRecord()
+            return
 
         try:
             length = int(first5)
         except ValueError:
-            raise RecordLengthInvalid
+            self._current_exception = exceptions.RecordLengthInvalid()
+            return
 
         chunk = self.file_handle.read(length - 5)
         chunk = first5 + chunk
         self._current_chunk = chunk
-        self._current_exception = None
+
+        if len(self._current_chunk) < length:
+            self._current_exception = exceptions.TruncatedRecord()
+            return
+
+        if self._current_chunk[-1] != ord(END_OF_RECORD):
+            self._current_exception = exceptions.EndOfRecordNotFound()
+            return
+
         try:
-            record = Record(
+            return Record(
                 chunk,
                 to_unicode=self.to_unicode,
                 force_utf8=self.force_utf8,
@@ -168,16 +183,11 @@ class MARCReader(Reader):
                 utf8_handling=self.utf8_handling,
                 file_encoding=self.file_encoding,
             )
-        except (PymarcException, UnicodeDecodeError, ValueError) as ex:
-            if self.permissive:
-                self._current_exception = ex
-                record = None
-            else:
-                raise ex
-        return record
+        except Exception as ex:
+            self._current_exception = ex
 
 
-def map_records(f, *files):
+def map_records(f: Callable, *files: BytesIO) -> None:
     """Applies a given function to each record in a batch.
 
     You can pass in multiple batches.
@@ -195,34 +205,41 @@ def map_records(f, *files):
 class JSONReader(Reader):
     """JSON Reader."""
 
-    def __init__(self, marc_target, encoding="utf-8", stream=False):
+    file_handle: IO
+
+    def __init__(
+        self,
+        marc_target: Union[bytes, str],
+        encoding: str = "utf-8",
+        stream: bool = False,
+    ) -> None:
         """The constructor to which you can pass either raw marc or a file-like object.
 
         Basically the argument you pass in should be raw JSON in transmission format or
         an object that responds to read().
         """
         self.encoding = encoding
-        if hasattr(marc_target, "read") and callable(marc_target.read):
+        if isinstance(marc_target, IOBase):
             self.file_handle = marc_target
         else:
-            if os.path.exists(marc_target):
+            if isinstance(marc_target, str) and os.path.exists(marc_target):
                 self.file_handle = open(marc_target, "r")
             else:
-                self.file_handle = StringIO(marc_target)
+                self.file_handle = StringIO(marc_target)  # type: ignore
         if stream:
             sys.stderr.write(
                 "Streaming not yet implemented, your data will be loaded into memory\n"
             )
         self.records = json.load(self.file_handle, strict=False)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         if hasattr(self.records, "__iter__") and not isinstance(self.records, dict):
             self.iter = iter(self.records)
         else:
             self.iter = iter([self.records])
         return self
 
-    def __next__(self):
+    def __next__(self) -> Iterator:
         jobj = next(self.iter)
         rec = Record()
         rec.leader = jobj["leader"]
@@ -230,7 +247,7 @@ class JSONReader(Reader):
             k, v = list(field.items())[0]
             if "subfields" in v and hasattr(v, "update"):
                 # flatten m-i-j dict to list in pymarc
-                subfields = []
+                subfields: list = []
                 for sub in v["subfields"]:
                     for code, value in sub.items():
                         subfields.extend((code, value))
